@@ -1,6 +1,7 @@
 // Iostreams base classes -*- C++ -*-
 
-// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+// 2006, 2007, 2008
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -28,26 +29,27 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
-//
-// ISO C++ 14882: 27.4  Iostreams base classes
-//
-
 /** @file ios_base.h
  *  This is an internal header file, included by other library headers.
  *  You should not attempt to use it directly.
  */
 
+//
+// ISO C++ 14882: 27.4  Iostreams base classes
+//
+
 #ifndef _IOS_BASE_H
 #define _IOS_BASE_H 1
 
 #pragma GCC system_header
 
-#include <bits/atomicity.h>
+#include <ext/atomicity.h>
 #include <bits/localefwd.h>
 #include <bits/locale_classes.h>
+#include <cstdio>  // For SEEK_CUR, SEEK_END
+
+_GLIBCXX_BEGIN_NAMESPACE(std)
 
-namespace std
-{
   // The following definitions of bitmask types are enums, not ints,
   // as permitted (but not required) in the standard, in order to provide
   // better type safety in iostream calls.  A side effect is that
@@ -450,9 +452,7 @@ namespace std
   protected:
     //@{
     /**
-     *  @if maint
      *  ios_base data members (doc me)
-     *  @endif
     */
     streamsize		_M_precision;
     streamsize		_M_width;
@@ -476,12 +476,12 @@ namespace std
       : _M_next(__cb), _M_fn(__fn), _M_index(__index), _M_refcount(0) { }
 
       void
-      _M_add_reference() { __gnu_cxx::__atomic_add(&_M_refcount, 1); }
+      _M_add_reference() { __gnu_cxx::__atomic_add_dispatch(&_M_refcount, 1); }
 
       // 0 => OK to delete.
       int
       _M_remove_reference() 
-      { return __gnu_cxx::__exchange_and_add(&_M_refcount, -1); }
+      { return __gnu_cxx::__exchange_and_add_dispatch(&_M_refcount, -1); }
     };
 
      _Callback_list*	_M_callbacks;
@@ -544,8 +544,9 @@ namespace std
      *  @brief  Access to format flags.
      *  @return  The format control flags for both input and output.
     */
-    inline fmtflags
-    flags() const { return _M_flags; }
+    fmtflags
+    flags() const
+    { return _M_flags; }
 
     /**
      *  @brief  Setting new format flags all at once.
@@ -554,7 +555,7 @@ namespace std
      *
      *  This function overwrites all the format flags with @a fmtfl.
     */
-    inline fmtflags
+    fmtflags
     flags(fmtflags __fmtfl)
     {
       fmtflags __old = _M_flags;
@@ -570,7 +571,7 @@ namespace std
      *  This function sets additional flags in format control.  Flags that
      *  were previously set remain set.
     */
-    inline fmtflags
+    fmtflags
     setf(fmtflags __fmtfl)
     {
       fmtflags __old = _M_flags;
@@ -587,7 +588,7 @@ namespace std
      *  This function clears @a mask in the format flags, then sets
      *  @a fmtfl @c & @a mask.  An example mask is @c ios_base::adjustfield.
     */
-    inline fmtflags
+    fmtflags
     setf(fmtflags __fmtfl, fmtflags __mask)
     {
       fmtflags __old = _M_flags;
@@ -602,27 +603,27 @@ namespace std
      *
      *  This function clears @a mask in the format flags.
     */
-    inline void
-    unsetf(fmtflags __mask) { _M_flags &= ~__mask; }
+    void
+    unsetf(fmtflags __mask)
+    { _M_flags &= ~__mask; }
 
     /**
      *  @brief  Flags access.
      *  @return  The precision to generate on certain output operations.
      *
-     *  @if maint
      *  Be careful if you try to give a definition of "precision" here; see
      *  DR 189.
-     *  @endif
     */
-    inline streamsize
-    precision() const { return _M_precision; }
+    streamsize
+    precision() const
+    { return _M_precision; }
 
     /**
      *  @brief  Changing flags.
      *  @param  prec  The new precision value.
      *  @return  The previous value of precision().
     */
-    inline streamsize
+    streamsize
     precision(streamsize __prec)
     {
       streamsize __old = _M_precision;
@@ -636,15 +637,16 @@ namespace std
      *
      *  "Minimum field width" refers to the number of characters.
     */
-    inline streamsize
-    width() const { return _M_width; }
+    streamsize
+    width() const
+    { return _M_width; }
 
     /**
      *  @brief  Changing flags.
      *  @param  wide  The new width value.
      *  @return  The previous value of width().
     */
-    inline streamsize
+    streamsize
     width(streamsize __wide)
     {
       streamsize __old = _M_width;
@@ -686,8 +688,9 @@ namespace std
      *  returns @c loc.  Otherwise, it returns a copy of @c std::locale(),
      *  the global C++ locale.
     */
-    inline locale
-    getloc() const { return _M_ios_locale; }
+    locale
+    getloc() const
+    { return _M_ios_locale; }
 
     /**
      *  @brief  Locale access
@@ -696,8 +699,9 @@ namespace std
      *  Like getloc above, but returns a reference instead of
      *  generating a copy.
     */
-    inline const locale&
-    _M_getloc() const { return _M_ios_locale; }
+    const locale&
+    _M_getloc() const
+    { return _M_ios_locale; }
 
     // [27.4.2.5] ios_base storage functions
     /**
@@ -730,7 +734,7 @@ namespace std
      *  obtain an index that is safe to use.  Also note that since the array
      *  can grow dynamically, it is not safe to hold onto the reference.
     */
-    inline long&
+    long&
     iword(int __ix)
     {
       _Words& __word = (__ix < _M_word_size)
@@ -751,7 +755,7 @@ namespace std
      *  obtain an index that is safe to use.  Also note that since the array
      *  can grow dynamically, it is not safe to hold onto the reference.
     */
-    inline void*&
+    void*&
     pword(int __ix)
     {
       _Words& __word = (__ix < _M_word_size)
@@ -895,7 +899,7 @@ namespace std
      return __base;
   }
 
-  // [27.4.5.2] adjustfield anipulators
+  // [27.4.5.2] adjustfield manipulators
   /// Calls base.setf(ios_base::internal, ios_base::adjustfield).
   inline ios_base&
   internal(ios_base& __base)
@@ -920,7 +924,7 @@ namespace std
     return __base;
   }
 
-  // [27.4.5.3] basefield anipulators
+  // [27.4.5.3] basefield manipulators
   /// Calls base.setf(ios_base::dec, ios_base::basefield).
   inline ios_base&
   dec(ios_base& __base)
@@ -945,7 +949,7 @@ namespace std
     return __base;
   }
 
-  // [27.4.5.4] floatfield anipulators
+  // [27.4.5.4] floatfield manipulators
   /// Calls base.setf(ios_base::fixed, ios_base::floatfield).
   inline ios_base&
   fixed(ios_base& __base)
@@ -961,7 +965,8 @@ namespace std
     __base.setf(ios_base::scientific, ios_base::floatfield);
     return __base;
   }
-} // namespace std
+
+_GLIBCXX_END_NAMESPACE
 
 #endif /* _IOS_BASE_H */
 
