@@ -13,11 +13,13 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.8.1.4 Overridden virtual functions
 
+// { dg-require-fileio "" }
+
 #include <locale>
 #include <fstream>
 #include <testsuite_hooks.h>
@@ -55,14 +57,17 @@ void test01()
   fb.sbumpc();
   fb.sbumpc();
 
-  // Check that seekoff calls codecvt::length with the correct state.
-  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::cur);
+  // Check that seekoff resets the state when seeking to beginning.
+  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::beg);
   VERIFY( pos != traits_type::pos_type(traits_type::off_type(-1)) );
 
   traits_type::int_type c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
-			  pod_uchar::from<char>('a')) );
+			  pod_uchar::from<char>(0xff)) );
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
   fb.sbumpc();
   fb.sbumpc();
   c = fb.sbumpc();
