@@ -1,4 +1,4 @@
-// Copyright (C) 2003 Free Software Foundation
+// Copyright (C) 2003, 2004 Free Software Foundation
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -13,7 +13,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.4.2.1 - Types [lib.ios.types]
@@ -23,7 +23,7 @@
 
 void test01()
 {
-  std::streamsize ssize; 
+  std::streamsize ssize = 0; 
   std::streamoff soff;
 
   // No signed/unsigned warnings.
