@@ -18,29 +18,31 @@
 // Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
+// { dg-require-fileio "" }
+
 #include <ext/stdio_sync_filebuf.h>
 #include <testsuite_hooks.h>
 
 // libstdc++/12048
-void test03()
+void test04()
 {
   bool test __attribute__((unused)) = true;
   const char* name = "cin_unget-1.txt";
 
   std::FILE* file = std::fopen(name, "r");
   __gnu_cxx::stdio_sync_filebuf<char> sbuf(file);
-  int c1 = sbuf.sbumpc();
-  VERIFY( c1 != EOF );
+  char buf[2];
+  VERIFY( sbuf.sgetn(buf, 2) == 2 );
   int c2 = sbuf.sungetc();
   VERIFY( c2 != EOF );
-  int c3 = std::fgetc(file);
-  VERIFY( c3 == c1 );
+  int c3 = sbuf.sbumpc();
+  VERIFY( c3 == std::char_traits<char>::to_int_type(buf[1]) );
 
   std::fclose(file);
 }
 
 int main ()
 {
-  test03();
+  test04();
   return 0;
 }
