@@ -1,6 +1,6 @@
 // 2004-10-20  Benjamin Kosnik  <bkoz@redhat.com>
 //
-// Copyright (C) 2004 Free Software Foundation, Inc.
+// Copyright (C) 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,7 +15,7 @@
 //
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 6.2.2 Class template array
@@ -30,9 +30,9 @@ test01()
     const size_t len = 5;
     typedef std::tr1::array<int, len> array_type;
     bool test __attribute__((unused)) = true;
-    array_type a = { 0, 1, 2, 3, 4 };
-    
-    VERIFY( a.size() == len );
+    array_type a = { { 0, 1, 2, 3, 4 } };
+
+    VERIFY( a.max_size() == len );
   }
 
   {
@@ -40,8 +40,8 @@ test01()
     typedef std::tr1::array<int, len> array_type;
     bool test __attribute__((unused)) = true;
     array_type a;
-    
-    VERIFY( a.size() == len );
+
+    VERIFY( a.max_size() == len );
   }
 }
 
@@ -50,4 +50,3 @@ int main()
   test01();
   return 0;
 }
-
